# This file was auto-generated by Fern from our API Definition.

import typing

from ..commons.types.action_parameter import ActionParameter
from ..commons.types.action_response import ActionResponse
from ..commons.types.entity_id import EntityId
from ..commons.types.entity_id_base import EntityIdBase
from ..commons.types.llm_inclusion_status import LlmInclusionStatus
from ..commons.types.precondition import Precondition
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawActionsClient, RawActionsClient
from .types.action_field import ActionField
from .types.action_filter import ActionFilter
from .types.actions_response import ActionsResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ActionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawActionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawActionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawActionsClient
        """
        return self._raw_client

    def search(
        self,
        *,
        sort: typing.Optional[ActionField] = OMIT,
        filter: typing.Optional[ActionFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ActionsResponse:
        """
        Parameters
        ----------
        sort : typing.Optional[ActionField]

        filter : typing.Optional[ActionFilter]

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ActionsResponse

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.actions.search()
        """
        _response = self._raw_client.search(
            sort=sort, filter=filter, page=page, size=size, sort_desc=sort_desc, request_options=request_options
        )
        return _response.data

    def create_or_update(
        self,
        *,
        action_id: EntityIdBase,
        name: str,
        description: str,
        user_interaction_required: bool,
        user_form_parameters: typing.Sequence[ActionParameter],
        button_name: typing.Optional[str] = OMIT,
        precondition: typing.Optional[Precondition] = OMIT,
        language: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ActionResponse:
        """
        Update an action or create it if it doesn't exist

        Parameters
        ----------
        action_id : EntityIdBase
            ID that uniquely identifies this action

        name : str
            The name of the action. This is displayed to the end user as part of forms when user interaction is required. It is also used to help Maven decide if the action is relevant to a conversation.

        description : str
            The description of the action. Must be less than 1024 characters. This helps Maven decide if the action is relevant to a conversation and is not displayed directly to the end user. Descriptions are used by the LLM.

        user_interaction_required : bool
            Whether the action requires user interaction to execute. If false, and all of the required action parameters are known, the LLM may call the action automatically. If true, an conversations ask call will return a BotActionFormResponse which must be submitted by an API caller. API callers must display a button with the buttonName label to confirm the user's intent.

        user_form_parameters : typing.Sequence[ActionParameter]
            The parameters that the action uses as input. An action will only be executed when all of the required parameters are provided. During execution, actions all have access to the full Conversation and User objects. Parameter values may be inferred from the user's conversation by the LLM.

        button_name : typing.Optional[str]
            When user interaction is required, the name of the button that is shown to the end user to confirm execution of the action. Defaults to "Submit" if not supplied.

        precondition : typing.Optional[Precondition]
            The preconditions that must be met for an action to be relevant to a conversation. Can be used to restrict actions to certain types of users.

        language : typing.Optional[str]
            The ISO 639-1 code for the language used in all fields of this action. Will be derived using the description's text if not specified.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ActionResponse

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.commons import EntityIdBase, Precondition_Group, Precondition_User

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.actions.create_or_update(
            action_id=EntityIdBase(
                reference_id="get-balance",
            ),
            name="Get the user's balance",
            description="This action calls an API to get the user's current balance.",
            user_interaction_required=False,
            user_form_parameters=[],
            precondition=Precondition_Group(
                operator="AND",
                preconditions=[
                    Precondition_User(
                        key="userKey",
                    ),
                    Precondition_User(
                        key="userKey2",
                    ),
                ],
            ),
            language="en",
        )
        """
        _response = self._raw_client.create_or_update(
            action_id=action_id,
            name=name,
            description=description,
            user_interaction_required=user_interaction_required,
            user_form_parameters=user_form_parameters,
            button_name=button_name,
            precondition=precondition,
            language=language,
            request_options=request_options,
        )
        return _response.data

    def get(
        self,
        action_reference_id: str,
        *,
        app_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ActionResponse:
        """
        Get an action by its supplied ID

        Parameters
        ----------
        action_reference_id : str
            The reference ID of the action to get. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the action to get. If not provided the ID of the calling app will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ActionResponse

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.actions.get(
            action_reference_id="get-balance",
        )
        """
        _response = self._raw_client.get(action_reference_id, app_id=app_id, request_options=request_options)
        return _response.data

    def patch(
        self,
        action_reference_id: str,
        *,
        app_id: typing.Optional[str] = OMIT,
        instructions: typing.Optional[str] = OMIT,
        llm_inclusion_status: typing.Optional[LlmInclusionStatus] = OMIT,
        segment_id: typing.Optional[EntityId] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ActionResponse:
        """
        Update mutable action fields

        The `appId` field can be provided to update an action owned by a different app.
        All other fields will overwrite the existing value on the action only if provided.

        Parameters
        ----------
        action_reference_id : str
            The reference ID of the action to patch.

        app_id : typing.Optional[str]
            The App ID of the action to patch. If not provided the ID of the calling app will be used.

        instructions : typing.Optional[str]
            The instructions given to the LLM when determining whether to execute the action.

        llm_inclusion_status : typing.Optional[LlmInclusionStatus]
            Determines whether the action is sent to the LLM as part of a conversation.

        segment_id : typing.Optional[EntityId]
            The ID of the segment that must be matched for the action to be relevant to a conversation.
            A null value will remove the segment from the action, it will be available on all conversations.

            Segments are replacing inline preconditions - an action may not have both an inline precondition and a segment.
            Inline precondition support will be removed in a future release.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ActionResponse

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.commons import EntityId

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.actions.patch(
            action_reference_id="get-balance",
            instructions="Use this action when the user asks about their account balance or remaining credits.",
            llm_inclusion_status="WHEN_RELEVANT",
            segment_id=EntityId(
                reference_id="premium-users",
                app_id="my-billing-system",
                organization_id="acme",
                agent_id="support",
                type="SEGMENT",
            ),
        )
        """
        _response = self._raw_client.patch(
            action_reference_id,
            app_id=app_id,
            instructions=instructions,
            llm_inclusion_status=llm_inclusion_status,
            segment_id=segment_id,
            request_options=request_options,
        )
        return _response.data

    def delete(self, action_reference_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete an action

        Parameters
        ----------
        action_reference_id : str
            The reference ID of the action to unregister. All other entity ID fields are inferred from the request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.actions.delete(
            action_reference_id="get-balance",
        )
        """
        _response = self._raw_client.delete(action_reference_id, request_options=request_options)
        return _response.data


class AsyncActionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawActionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawActionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawActionsClient
        """
        return self._raw_client

    async def search(
        self,
        *,
        sort: typing.Optional[ActionField] = OMIT,
        filter: typing.Optional[ActionFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ActionsResponse:
        """
        Parameters
        ----------
        sort : typing.Optional[ActionField]

        filter : typing.Optional[ActionFilter]

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ActionsResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.actions.search()


        asyncio.run(main())
        """
        _response = await self._raw_client.search(
            sort=sort, filter=filter, page=page, size=size, sort_desc=sort_desc, request_options=request_options
        )
        return _response.data

    async def create_or_update(
        self,
        *,
        action_id: EntityIdBase,
        name: str,
        description: str,
        user_interaction_required: bool,
        user_form_parameters: typing.Sequence[ActionParameter],
        button_name: typing.Optional[str] = OMIT,
        precondition: typing.Optional[Precondition] = OMIT,
        language: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ActionResponse:
        """
        Update an action or create it if it doesn't exist

        Parameters
        ----------
        action_id : EntityIdBase
            ID that uniquely identifies this action

        name : str
            The name of the action. This is displayed to the end user as part of forms when user interaction is required. It is also used to help Maven decide if the action is relevant to a conversation.

        description : str
            The description of the action. Must be less than 1024 characters. This helps Maven decide if the action is relevant to a conversation and is not displayed directly to the end user. Descriptions are used by the LLM.

        user_interaction_required : bool
            Whether the action requires user interaction to execute. If false, and all of the required action parameters are known, the LLM may call the action automatically. If true, an conversations ask call will return a BotActionFormResponse which must be submitted by an API caller. API callers must display a button with the buttonName label to confirm the user's intent.

        user_form_parameters : typing.Sequence[ActionParameter]
            The parameters that the action uses as input. An action will only be executed when all of the required parameters are provided. During execution, actions all have access to the full Conversation and User objects. Parameter values may be inferred from the user's conversation by the LLM.

        button_name : typing.Optional[str]
            When user interaction is required, the name of the button that is shown to the end user to confirm execution of the action. Defaults to "Submit" if not supplied.

        precondition : typing.Optional[Precondition]
            The preconditions that must be met for an action to be relevant to a conversation. Can be used to restrict actions to certain types of users.

        language : typing.Optional[str]
            The ISO 639-1 code for the language used in all fields of this action. Will be derived using the description's text if not specified.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ActionResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.commons import EntityIdBase, Precondition_Group, Precondition_User

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.actions.create_or_update(
                action_id=EntityIdBase(
                    reference_id="get-balance",
                ),
                name="Get the user's balance",
                description="This action calls an API to get the user's current balance.",
                user_interaction_required=False,
                user_form_parameters=[],
                precondition=Precondition_Group(
                    operator="AND",
                    preconditions=[
                        Precondition_User(
                            key="userKey",
                        ),
                        Precondition_User(
                            key="userKey2",
                        ),
                    ],
                ),
                language="en",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_or_update(
            action_id=action_id,
            name=name,
            description=description,
            user_interaction_required=user_interaction_required,
            user_form_parameters=user_form_parameters,
            button_name=button_name,
            precondition=precondition,
            language=language,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self,
        action_reference_id: str,
        *,
        app_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ActionResponse:
        """
        Get an action by its supplied ID

        Parameters
        ----------
        action_reference_id : str
            The reference ID of the action to get. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the action to get. If not provided the ID of the calling app will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ActionResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.actions.get(
                action_reference_id="get-balance",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(action_reference_id, app_id=app_id, request_options=request_options)
        return _response.data

    async def patch(
        self,
        action_reference_id: str,
        *,
        app_id: typing.Optional[str] = OMIT,
        instructions: typing.Optional[str] = OMIT,
        llm_inclusion_status: typing.Optional[LlmInclusionStatus] = OMIT,
        segment_id: typing.Optional[EntityId] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ActionResponse:
        """
        Update mutable action fields

        The `appId` field can be provided to update an action owned by a different app.
        All other fields will overwrite the existing value on the action only if provided.

        Parameters
        ----------
        action_reference_id : str
            The reference ID of the action to patch.

        app_id : typing.Optional[str]
            The App ID of the action to patch. If not provided the ID of the calling app will be used.

        instructions : typing.Optional[str]
            The instructions given to the LLM when determining whether to execute the action.

        llm_inclusion_status : typing.Optional[LlmInclusionStatus]
            Determines whether the action is sent to the LLM as part of a conversation.

        segment_id : typing.Optional[EntityId]
            The ID of the segment that must be matched for the action to be relevant to a conversation.
            A null value will remove the segment from the action, it will be available on all conversations.

            Segments are replacing inline preconditions - an action may not have both an inline precondition and a segment.
            Inline precondition support will be removed in a future release.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ActionResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.commons import EntityId

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.actions.patch(
                action_reference_id="get-balance",
                instructions="Use this action when the user asks about their account balance or remaining credits.",
                llm_inclusion_status="WHEN_RELEVANT",
                segment_id=EntityId(
                    reference_id="premium-users",
                    app_id="my-billing-system",
                    organization_id="acme",
                    agent_id="support",
                    type="SEGMENT",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.patch(
            action_reference_id,
            app_id=app_id,
            instructions=instructions,
            llm_inclusion_status=llm_inclusion_status,
            segment_id=segment_id,
            request_options=request_options,
        )
        return _response.data

    async def delete(
        self, action_reference_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Delete an action

        Parameters
        ----------
        action_reference_id : str
            The reference ID of the action to unregister. All other entity ID fields are inferred from the request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.actions.delete(
                action_reference_id="get-balance",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(action_reference_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...commons.types.llm_persona import LlmPersona
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class AgentPrompting(UniversalBaseModel):
    persona: LlmPersona = pydantic.Field()
    """
    The overall persona of the agent.
    """

    additional_prompt_text: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="additionalPromptText")
    ] = pydantic.Field(default=None)
    """
    Additional text directly appended to the prompt. This field is replaced by Knowledge Documents with `llmInclusionsStatus` set to `ALWAYS`.
    """

    category_generation_prompt_text: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="categoryGenerationPromptText")
    ] = pydantic.Field(default=None)
    """
    LLM prompt for category generation. 
    
    Use this to specify a fixed set of categories that an LLM should choose from when analyzing a conversation. 
    When not set uses a default prompt which gives the LLM flexibility to determine appropriate categories on its own.
    """

    content_safety_violation_response_prompt_text: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="contentSafetyViolationResponsePromptText")
    ] = pydantic.Field(default=None)
    """
    LLM prompt for generating a response when the user's question has been detected as unsafe. 
    
    Use this to provide custom LLM generated replies when dealing with content safety violations.
    When not set, the system fallback message will be used and replies will not be personalized by the LLM as it will not be involved.
    """

    reject_questions_without_knowledge: typing_extensions.Annotated[
        bool, FieldMetadata(alias="rejectQuestionsWithoutKnowledge")
    ] = pydantic.Field()
    """
    Return the system fallback message on all questions that have no relevant knowledge bases or actions.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel, update_forward_refs


class KnowledgeBaseProperties(UniversalBaseModel):
    name: str = pydantic.Field()
    """
    The name of the knowledge base
    """

    precondition: typing.Optional["Precondition"] = pydantic.Field(default=None)
    """
    The preconditions that must be met for knowledge base be relevant to a conversation. Can be used to restrict knowledge bases to certain types of users.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from ...commons.types.precondition_group import PreconditionGroup  # noqa: E402, F401, I001
from ...commons.types.precondition import Precondition  # noqa: E402, F401, I001

update_forward_refs(KnowledgeBaseProperties)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .action_form_field import ActionFormField
from .entity_id_without_agent import EntityIdWithoutAgent


class BotActionFormResponse(UniversalBaseModel):
    """
    This response should be rendered as a form which users can submit. Upon submission call the `submitActionForm` API.
    """

    id: str = pydantic.Field()
    """
    The ID to use when submitting the form via the `submitActionForm` API.
    """

    action_id: typing_extensions.Annotated[EntityIdWithoutAgent, FieldMetadata(alias="actionId")] = pydantic.Field()
    """
    The ID of the action that will be executed when the form is submitted.
    """

    form_label: typing_extensions.Annotated[str, FieldMetadata(alias="formLabel")] = pydantic.Field()
    """
    Text which should be displayed to the user at the top of the form. Provided in the user's language.
    """

    fields: typing.List[ActionFormField] = pydantic.Field()
    """
    The fields that should be displayed within the form.
    """

    submit_label: typing_extensions.Annotated[str, FieldMetadata(alias="submitLabel")] = pydantic.Field()
    """
    Text that should be displayed to the user on the submit button. Provided in the user's language.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    ActionFormAttachment,
    ActionFormRequestParamValue,
    AskObjectRequest,
    AskRequest,
    AskStreamActionEvent,
    AskStreamChartEvent,
    AskStreamEndEvent,
    AskStreamMetadataEvent,
    AskStreamOAuthButtonEvent,
    AskStreamStartEvent,
    AskStreamTextEvent,
    CategorizationResponse,
    ConversationField,
    ConversationFilter,
    ConversationMessageRequest,
    ConversationMetadata,
    ConversationPatchRequest,
    ConversationRequest,
    ConversationsResponse,
    ConversationsSearchRequest,
    DeliverConversationMessageRequest,
    DeliverMessageRequest,
    DeliverMessageRequest_Conversation,
    DeliverMessageRequest_User,
    DeliverMessageResponse,
    DeliverUserMessageRequest,
    DeliveryStatus,
    FeedbackField,
    FeedbackFilter,
    FeedbackRequest,
    NumericConversationField,
    ObjectStreamResponse,
    ObjectStreamResponse_End,
    ObjectStreamResponse_Start,
    ObjectStreamResponse_Text,
    ResolutionStatus,
    SimulationFilter,
    StreamResponse,
    StreamResponse_Action,
    StreamResponse_Chart,
    StreamResponse_End,
    StreamResponse_Metadata,
    StreamResponse_OauthButton,
    StreamResponse_Start,
    StreamResponse_Text,
    SubmitActionFormRequest,
    UpdateMetadataRequest,
)

__all__ = [
    "ActionFormAttachment",
    "ActionFormRequestParamValue",
    "AskObjectRequest",
    "AskRequest",
    "AskStreamActionEvent",
    "AskStreamChartEvent",
    "AskStreamEndEvent",
    "AskStreamMetadataEvent",
    "AskStreamOAuthButtonEvent",
    "AskStreamStartEvent",
    "AskStreamTextEvent",
    "CategorizationResponse",
    "ConversationField",
    "ConversationFilter",
    "ConversationMessageRequest",
    "ConversationMetadata",
    "ConversationPatchRequest",
    "ConversationRequest",
    "ConversationsResponse",
    "ConversationsSearchRequest",
    "DeliverConversationMessageRequest",
    "DeliverMessageRequest",
    "DeliverMessageRequest_Conversation",
    "DeliverMessageRequest_User",
    "DeliverMessageResponse",
    "DeliverUserMessageRequest",
    "DeliveryStatus",
    "FeedbackField",
    "FeedbackFilter",
    "FeedbackRequest",
    "NumericConversationField",
    "ObjectStreamResponse",
    "ObjectStreamResponse_End",
    "ObjectStreamResponse_Start",
    "ObjectStreamResponse_Text",
    "ResolutionStatus",
    "SimulationFilter",
    "StreamResponse",
    "StreamResponse_Action",
    "StreamResponse_Chart",
    "StreamResponse_End",
    "StreamResponse_Metadata",
    "StreamResponse_OauthButton",
    "StreamResponse_Start",
    "StreamResponse_Text",
    "SubmitActionFormRequest",
    "UpdateMetadataRequest",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ...commons.types.entity_id_base import EntityIdBase
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .knowledge_base_properties import KnowledgeBaseProperties


class KnowledgeBaseRequest(KnowledgeBaseProperties):
    """
    Examples
    --------
    from mavenagi.commons import EntityIdBase
    from mavenagi.knowledge import KnowledgeBaseRequest

    KnowledgeBaseRequest(
        knowledge_base_id=EntityIdBase(
            reference_id="help-center",
        ),
        name="Help center",
    )
    """

    knowledge_base_id: typing_extensions.Annotated[EntityIdBase, FieldMetadata(alias="knowledgeBaseId")] = (
        pydantic.Field()
    )
    """
    ID that uniquely identifies this knowledge base
    """

    metadata: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    Metadata for the knowledge base.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from ...commons.types.precondition_group import PreconditionGroup  # noqa: E402, F401, I001



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .entity_id import EntityId
from .llm_persona import LlmPersona


class SimulationContext(UniversalBaseModel):
    additional_prompt_text: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="additionalPromptText")
    ] = pydantic.Field(default=None)
    """
    If provided, overrides the agent's default additional prompt text during the simulation. 
    Note that this field is provided for backwards compatibility and will be removed in a future release. 
    Instead please use the `availableKnowledgeBases` field to include a knowledge base with a document `llmInclusionStatus` set to `ALWAYS`.
    """

    persona: typing.Optional[LlmPersona] = pydantic.Field(default=None)
    """
    The persona to use during the simulation. If not provided, the agent's default persona will be used.
    """

    available_knowledge_bases: typing_extensions.Annotated[
        typing.Optional[typing.List[EntityId]], FieldMetadata(alias="availableKnowledgeBases")
    ] = pydantic.Field(default=None)
    """
    If provided, knowledge search will be restricted to the provided list of knowledge bases. Otherwise, all active knowledge bases will be used. An empty list means no knowledge bases will be used.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .attachment_status import AttachmentStatus
from .base_attachment import BaseAttachment


class AttachmentResponse(BaseAttachment):
    url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The URL to access the attachment, The URL will be valid for 20 minutes. Only present when status is `ACCEPTED`.
    """

    size_bytes: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="sizeBytes")] = pydantic.Field(
        default=None
    )
    """
    The size of the attachment in bytes
    """

    status: AttachmentStatus = pydantic.Field()
    """
    The processing status of the attachment.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .survey_info import SurveyInfo


class FeedbackInfo(UniversalBaseModel):
    rating: typing.Optional[float] = pydantic.Field(default=None)
    """
    The rating of the feedback as a ratio [0.0, 1.0]
    """

    thumb_up: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="thumbUp")] = pydantic.Field(
        default=None
    )
    """
    Whether the feedback was marked as a thumbs up
    """

    survey: typing.Optional[SurveyInfo] = pydantic.Field(default=None)
    """
    A question and answer associated with the feedback
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ...commons.types.action_response import ActionResponse
from ...commons.types.page import Page
from ...core.pydantic_utilities import IS_PYDANTIC_V2, update_forward_refs


class ActionsResponse(Page):
    actions: typing.List[ActionResponse] = pydantic.Field()
    """
    The actions that match the search criteria
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from ...commons.types.precondition_group import PreconditionGroup  # noqa: E402, F401, I001

update_forward_refs(ActionsResponse)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...commons.types.attachment_request import AttachmentRequest
from ...commons.types.entity_id_base import EntityIdBase
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class AskRequest(UniversalBaseModel):
    """
    Examples
    --------
    from mavenagi.commons import AttachmentRequest, EntityIdBase
    from mavenagi.conversation import AskRequest

    AskRequest(
        conversation_message_id=EntityIdBase(
            reference_id="message-0",
        ),
        user_id=EntityIdBase(
            reference_id="user-0",
        ),
        text="How do I reset my password?",
        attachments=[
            AttachmentRequest(
                type="image/png",
                content="iVBORw0KGgo...",
            )
        ],
        transient_data={"userToken": "abcdef123", "queryApiKey": "foobar456"},
        timezone="America/New_York",
    )
    """

    conversation_message_id: typing_extensions.Annotated[EntityIdBase, FieldMetadata(alias="conversationMessageId")] = (
        pydantic.Field()
    )
    """
    Externally supplied ID to uniquely identify this message within the conversation. If a message with this ID already exists it will be reused and will not be updated.
    """

    user_id: typing_extensions.Annotated[EntityIdBase, FieldMetadata(alias="userId")] = pydantic.Field()
    """
    Externally supplied ID to uniquely identify the user that created this message
    """

    text: str = pydantic.Field()
    """
    The text of the message
    """

    attachments: typing.Optional[typing.List[AttachmentRequest]] = pydantic.Field(default=None)
    """
    The attachments to the message. Image attachments will be sent to the LLM as additional data.
    Non-image attachments can be stored and downloaded from the API but will not be sent to the LLM.
    """

    transient_data: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]], FieldMetadata(alias="transientData")
    ] = pydantic.Field(default=None)
    """
    Transient data which the Maven platform will not persist. This data will only be forwarded to actions taken by this ask request. For example, one may put in user tokens as transient data.
    """

    timezone: typing.Optional[str] = pydantic.Field(default=None)
    """
    IANA timezone identifier (e.g. "America/New_York", "Europe/London") to be used for time-based operations in the conversation.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...commons.types.page import Page
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .agent_user import AgentUser


class AgentUserSearchResponse(Page):
    agent_users: typing_extensions.Annotated[typing.List[AgentUser], FieldMetadata(alias="agentUsers")] = (
        pydantic.Field()
    )
    """
    The agent users that match the search criteria
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...commons.types.llm_inclusion_status import LlmInclusionStatus
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class ActionFilter(UniversalBaseModel):
    instructions: typing.Optional[str] = pydantic.Field(default=None)
    """
    Filter by instructions
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Filter by action name
    """

    llm_inclusion_statuses: typing_extensions.Annotated[
        typing.Optional[typing.List[LlmInclusionStatus]], FieldMetadata(alias="llmInclusionStatuses")
    ] = pydantic.Field(default=None)
    """
    Filter by LLM inclusion status
    """

    app_ids: typing_extensions.Annotated[typing.Optional[typing.List[str]], FieldMetadata(alias="appIds")] = (
        pydantic.Field(default=None)
    )
    """
    Filter by app IDs
    """

    user_interaction_required: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="userInteractionRequired")
    ] = pydantic.Field(default=None)
    """
    Filter by user interaction required
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .entity_id import EntityId
from .inbox_item_severity import InboxItemSeverity
from .inbox_item_status import InboxItemStatus


class InboxItemBase(UniversalBaseModel):
    id: EntityId = pydantic.Field()
    """
    Unique identifier for the inbox item.
    """

    created_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="createdAt")] = pydantic.Field()
    """
    Timestamp when the inbox item was created.
    """

    updated_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="updatedAt")] = pydantic.Field()
    """
    Timestamp when the inbox item was last updated.
    """

    status: InboxItemStatus = pydantic.Field()
    """
    Status of the inbox item.
    """

    severity: InboxItemSeverity = pydantic.Field()
    """
    Severity of the inbox item.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    ActionBase,
    ActionEnumOption,
    ActionExecutionParamValue,
    ActionExecutionParamValue_Attachment,
    ActionExecutionParamValue_Primitive,
    ActionFormField,
    ActionOAuthConfiguration,
    ActionParameter,
    ActionParameterType,
    ActionResponse,
    AppPrecondition,
    AppUser,
    AppUserIdentifier,
    AppUserIdentifyingPropertyType,
    AppUserRequest,
    AppUserResponse,
    AttachmentRequest,
    AttachmentResponse,
    AttachmentStatus,
    BaseAttachment,
    BaseConversationResponse,
    BasePaginatedRequest,
    BaseSearchRequest,
    BotActionFormResponse,
    BotChartResponse,
    BotConversationMessageType,
    BotLogic,
    BotLogicActionExecutedDetail,
    BotLogicActionItem,
    BotLogicActionReviewedDetail,
    BotLogicFormSubmissionItem,
    BotLogicItem,
    BotLogicItem_Actions,
    BotLogicItem_Form,
    BotLogicItem_Knowledge,
    BotLogicItem_Safety,
    BotLogicItem_Segments,
    BotLogicItem_User,
    BotLogicKnowledgeDetail,
    BotLogicKnowledgeItem,
    BotLogicSafetyItem,
    BotLogicSegmentsItem,
    BotLogicUserItem,
    BotMessage,
    BotOAuthButtonResponse,
    BotObjectResponse,
    BotResponse,
    BotResponseMetadata,
    BotResponse_ActionForm,
    BotResponse_Chart,
    BotResponse_OauthButton,
    BotResponse_Object,
    BotResponse_Text,
    BotTextResponse,
    BrowserInfo,
    BrowserType,
    Capability,
    ChartSpecSchema,
    ContextInfo,
    ConversationAnalysis,
    ConversationExecutedActionPrecondition,
    ConversationInformation,
    ConversationMessageBase,
    ConversationMessageResponse,
    ConversationMessageResponse_Bot,
    ConversationMessageResponse_User,
    ConversationPrecondition,
    ConversationPrecondition_ActionExecuted,
    ConversationPrecondition_App,
    ConversationPrecondition_Metadata,
    ConversationPrecondition_ResponseConfig,
    ConversationPrecondition_Tags,
    ConversationPreview,
    ConversationResponse,
    ConversationSummary,
    DeviceInfo,
    DeviceType,
    DocumentInformation,
    EntityId,
    EntityIdBase,
    EntityIdFilter,
    EntityIdWithoutAgent,
    EntityResult,
    EntityType,
    ErrorMessage,
    EventBaseNoId,
    EventField,
    EventFilter,
    EventRequest,
    EventRequest_SystemEvent,
    EventRequest_UserEvent,
    EventResponse,
    EventResponse_SystemEvent,
    EventResponse_UserEvent,
    EventType,
    EventUserInfo,
    EventUserInfoBase,
    EventsSearchRequest,
    EventsSearchResponse,
    Feedback,
    FeedbackBase,
    FeedbackInfo,
    FeedbackType,
    GeoInfo,
    InboxItem,
    InboxItemBase,
    InboxItemCustom,
    InboxItemDuplicateDocuments,
    InboxItemFix,
    InboxItemFixAddDocument,
    InboxItemFixBase,
    InboxItemFixDeactivateDocument,
    InboxItemFix_AddDocument,
    InboxItemFix_DeactivateDocument,
    InboxItemMissingKnowledge,
    InboxItemSeverity,
    InboxItemStatus,
    InboxItemType,
    InboxItem_Custom,
    InboxItem_DuplicateDocuments,
    InboxItem_MissingKnowledge,
    IpInfo,
    LanguageInfo,
    LlmInclusionStatus,
    LlmPersona,
    MessageStatus,
    MetadataPrecondition,
    NovelSystemEvent,
    NovelSystemEventNoId,
    NovelUserEvent,
    NumberRange,
    OsInfo,
    OsType,
    Page,
    PageInfo,
    Precondition,
    PreconditionBase,
    PreconditionGroup,
    PreconditionGroupOperator,
    PreconditionOperator,
    Precondition_Conversation,
    Precondition_Group,
    Precondition_User,
    Quality,
    QualityReason,
    ResponseConfig,
    ResponseConfigPrecondition,
    ResponseLength,
    ScopedEntity,
    Sentiment,
    SessionInfo,
    SimulationContext,
    Source,
    SourceInfo,
    SourceType,
    SurveyInfo,
    SystemEvent,
    SystemEventName,
    TagsPrecondition,
    UserConversationMessageType,
    UserData,
    UserDataWithReference,
    UserEvent,
    UserEventName,
    UserMessage,
    UserMessageBase,
    UserMessageResponseState,
    VisibilityType,
)
from .errors import BadRequestError, NotFoundError, ServerError

__all__ = [
    "ActionBase",
    "ActionEnumOption",
    "ActionExecutionParamValue",
    "ActionExecutionParamValue_Attachment",
    "ActionExecutionParamValue_Primitive",
    "ActionFormField",
    "ActionOAuthConfiguration",
    "ActionParameter",
    "ActionParameterType",
    "ActionResponse",
    "AppPrecondition",
    "AppUser",
    "AppUserIdentifier",
    "AppUserIdentifyingPropertyType",
    "AppUserRequest",
    "AppUserResponse",
    "AttachmentRequest",
    "AttachmentResponse",
    "AttachmentStatus",
    "BadRequestError",
    "BaseAttachment",
    "BaseConversationResponse",
    "BasePaginatedRequest",
    "BaseSearchRequest",
    "BotActionFormResponse",
    "BotChartResponse",
    "BotConversationMessageType",
    "BotLogic",
    "BotLogicActionExecutedDetail",
    "BotLogicActionItem",
    "BotLogicActionReviewedDetail",
    "BotLogicFormSubmissionItem",
    "BotLogicItem",
    "BotLogicItem_Actions",
    "BotLogicItem_Form",
    "BotLogicItem_Knowledge",
    "BotLogicItem_Safety",
    "BotLogicItem_Segments",
    "BotLogicItem_User",
    "BotLogicKnowledgeDetail",
    "BotLogicKnowledgeItem",
    "BotLogicSafetyItem",
    "BotLogicSegmentsItem",
    "BotLogicUserItem",
    "BotMessage",
    "BotOAuthButtonResponse",
    "BotObjectResponse",
    "BotResponse",
    "BotResponseMetadata",
    "BotResponse_ActionForm",
    "BotResponse_Chart",
    "BotResponse_OauthButton",
    "BotResponse_Object",
    "BotResponse_Text",
    "BotTextResponse",
    "BrowserInfo",
    "BrowserType",
    "Capability",
    "ChartSpecSchema",
    "ContextInfo",
    "ConversationAnalysis",
    "ConversationExecutedActionPrecondition",
    "ConversationInformation",
    "ConversationMessageBase",
    "ConversationMessageResponse",
    "ConversationMessageResponse_Bot",
    "ConversationMessageResponse_User",
    "ConversationPrecondition",
    "ConversationPrecondition_ActionExecuted",
    "ConversationPrecondition_App",
    "ConversationPrecondition_Metadata",
    "ConversationPrecondition_ResponseConfig",
    "ConversationPrecondition_Tags",
    "ConversationPreview",
    "ConversationResponse",
    "ConversationSummary",
    "DeviceInfo",
    "DeviceType",
    "DocumentInformation",
    "EntityId",
    "EntityIdBase",
    "EntityIdFilter",
    "EntityIdWithoutAgent",
    "EntityResult",
    "EntityType",
    "ErrorMessage",
    "EventBaseNoId",
    "EventField",
    "EventFilter",
    "EventRequest",
    "EventRequest_SystemEvent",
    "EventRequest_UserEvent",
    "EventResponse",
    "EventResponse_SystemEvent",
    "EventResponse_UserEvent",
    "EventType",
    "EventUserInfo",
    "EventUserInfoBase",
    "EventsSearchRequest",
    "EventsSearchResponse",
    "Feedback",
    "FeedbackBase",
    "FeedbackInfo",
    "FeedbackType",
    "GeoInfo",
    "InboxItem",
    "InboxItemBase",
    "InboxItemCustom",
    "InboxItemDuplicateDocuments",
    "InboxItemFix",
    "InboxItemFixAddDocument",
    "InboxItemFixBase",
    "InboxItemFixDeactivateDocument",
    "InboxItemFix_AddDocument",
    "InboxItemFix_DeactivateDocument",
    "InboxItemMissingKnowledge",
    "InboxItemSeverity",
    "InboxItemStatus",
    "InboxItemType",
    "InboxItem_Custom",
    "InboxItem_DuplicateDocuments",
    "InboxItem_MissingKnowledge",
    "IpInfo",
    "LanguageInfo",
    "LlmInclusionStatus",
    "LlmPersona",
    "MessageStatus",
    "MetadataPrecondition",
    "NotFoundError",
    "NovelSystemEvent",
    "NovelSystemEventNoId",
    "NovelUserEvent",
    "NumberRange",
    "OsInfo",
    "OsType",
    "Page",
    "PageInfo",
    "Precondition",
    "PreconditionBase",
    "PreconditionGroup",
    "PreconditionGroupOperator",
    "PreconditionOperator",
    "Precondition_Conversation",
    "Precondition_Group",
    "Precondition_User",
    "Quality",
    "QualityReason",
    "ResponseConfig",
    "ResponseConfigPrecondition",
    "ResponseLength",
    "ScopedEntity",
    "Sentiment",
    "ServerError",
    "SessionInfo",
    "SimulationContext",
    "Source",
    "SourceInfo",
    "SourceType",
    "SurveyInfo",
    "SystemEvent",
    "SystemEventName",
    "TagsPrecondition",
    "UserConversationMessageType",
    "UserData",
    "UserDataWithReference",
    "UserEvent",
    "UserEventName",
    "UserMessage",
    "UserMessageBase",
    "UserMessageResponseState",
    "VisibilityType",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .action_execution_param_value import ActionExecutionParamValue
from .bot_logic_action_executed_detail import BotLogicActionExecutedDetail
from .bot_logic_action_reviewed_detail import BotLogicActionReviewedDetail
from .bot_logic_knowledge_detail import BotLogicKnowledgeDetail
from .entity_id import EntityId
from .entity_id_without_agent import EntityIdWithoutAgent


class BotLogicItem_Knowledge(UniversalBaseModel):
    type: typing.Literal["knowledge"] = "knowledge"
    knowledge_reviewed: typing_extensions.Annotated[
        typing.List[BotLogicKnowledgeDetail], FieldMetadata(alias="knowledgeReviewed")
    ]
    knowledge_utilized: typing_extensions.Annotated[
        typing.List[BotLogicKnowledgeDetail], FieldMetadata(alias="knowledgeUtilized")
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class BotLogicItem_Actions(UniversalBaseModel):
    type: typing.Literal["actions"] = "actions"
    actions_reviewed: typing_extensions.Annotated[
        typing.List[BotLogicActionReviewedDetail], FieldMetadata(alias="actionsReviewed")
    ]
    actions_executed: typing_extensions.Annotated[
        typing.List[BotLogicActionExecutedDetail], FieldMetadata(alias="actionsExecuted")
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class BotLogicItem_Form(UniversalBaseModel):
    type: typing.Literal["form"] = "form"
    action_parameters: typing_extensions.Annotated[
        typing.Dict[str, ActionExecutionParamValue], FieldMetadata(alias="actionParameters")
    ]
    execution_result: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="executionResult")] = None
    execution_error: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="executionError")] = None
    action_id: typing_extensions.Annotated[EntityIdWithoutAgent, FieldMetadata(alias="actionId")]
    action_name: typing_extensions.Annotated[str, FieldMetadata(alias="actionName")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class BotLogicItem_Safety(UniversalBaseModel):
    type: typing.Literal["safety"] = "safety"
    safety_check_passed: typing_extensions.Annotated[bool, FieldMetadata(alias="safetyCheckPassed")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class BotLogicItem_User(UniversalBaseModel):
    type: typing.Literal["user"] = "user"
    user_data: typing_extensions.Annotated[typing.Dict[str, str], FieldMetadata(alias="userData")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class BotLogicItem_Segments(UniversalBaseModel):
    type: typing.Literal["segments"] = "segments"
    matching_segment_ids: typing_extensions.Annotated[typing.List[EntityId], FieldMetadata(alias="matchingSegmentIds")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


BotLogicItem = typing.Union[
    BotLogicItem_Knowledge,
    BotLogicItem_Actions,
    BotLogicItem_Form,
    BotLogicItem_Safety,
    BotLogicItem_User,
    BotLogicItem_Segments,
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .action_field import ActionField
from .action_filter import ActionFilter
from .action_request import ActionRequest
from .actions_response import ActionsResponse
from .actions_search_request import ActionsSearchRequest

__all__ = ["ActionField", "ActionFilter", "ActionRequest", "ActionsResponse", "ActionsSearchRequest"]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class BaseAttachment(UniversalBaseModel):
    type: str = pydantic.Field()
    """
    The mime-type of the attachment. Supported types are:
    - image/jpeg
    - image/jpg
    - image/png
    - image/gif
    - image/webp
    - application/pdf
    - text/plain
    - text/csv
    - application/vnd.openxmlformats-officedocument.wordprocessingml.document
    - application/vnd.openxmlformats-officedocument.spreadsheetml.sheet
    - application/vnd.openxmlformats-officedocument.presentationml.presentation
    - application/msword
    - application/vnd.ms-excel
    - application/vnd.ms-powerpoint
    - audio/aac
    - audio/mpeg
    - audio/mp4
    - audio/wav
    - audio/ogg
    - video/mp4
    - video/webm
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    An optional name for the attachment.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...commons.types.attachment_request import AttachmentRequest
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class ConversationPatchRequest(UniversalBaseModel):
    """
    Examples
    --------
    from mavenagi.conversation import ConversationPatchRequest

    ConversationPatchRequest(
        llm_enabled=True,
    )
    """

    app_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="appId")] = pydantic.Field(
        default=None
    )
    """
    The App ID of the conversation to patch. If not provided the ID of the calling app will be used.
    """

    open: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the conversation is able to receive asynchronous messages. Only valid for conversations with the `ASYNC` capability.
    """

    llm_enabled: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="llmEnabled")] = pydantic.Field(
        default=None
    )
    """
    Whether the LLM is enabled for this conversation.
    """

    attachments: typing.Optional[typing.List[AttachmentRequest]] = pydantic.Field(default=None)
    """
    A list of attachments to add to the conversation. Attachments can only be appended. Removal is not allowed.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .agent import Agent
from .agent_environment import AgentEnvironment
from .agent_field import AgentField
from .agent_filter import AgentFilter
from .agent_prompting import AgentPrompting
from .agents_search_request import AgentsSearchRequest
from .agents_search_response import AgentsSearchResponse
from .create_agent_request import CreateAgentRequest
from .pii_category import PiiCategory

__all__ = [
    "Agent",
    "AgentEnvironment",
    "AgentField",
    "AgentFilter",
    "AgentPrompting",
    "AgentsSearchRequest",
    "AgentsSearchResponse",
    "CreateAgentRequest",
    "PiiCategory",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..analytics.types.chart_response import ChartResponse
from ..analytics.types.conversation_chart_request import ConversationChartRequest
from ..analytics.types.conversation_column_definition import ConversationColumnDefinition
from ..analytics.types.conversation_group_by import ConversationGroupBy
from ..analytics.types.conversation_table_response import ConversationTableResponse
from ..analytics.types.time_interval import TimeInterval
from ..conversation.types.conversation_filter import ConversationFilter
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawOrganizationsClient, RawOrganizationsClient
from .types.organization import Organization

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class OrganizationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawOrganizationsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawOrganizationsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawOrganizationsClient
        """
        return self._raw_client

    def create(
        self,
        organization_reference_id: str,
        *,
        name: str,
        default_language: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Organization:
        """
        Create a new organization.

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        organization_reference_id : str
            The reference ID of the organization.

        name : str
            The name of the organization.

        default_language : str
            The default language for the organization in ISO 639-1 code format.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Organization

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.organizations.create(
            organization_reference_id="organizationReferenceId",
            name="name",
            default_language="defaultLanguage",
        )
        """
        _response = self._raw_client.create(
            organization_reference_id, name=name, default_language=default_language, request_options=request_options
        )
        return _response.data

    def get(
        self, organization_reference_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> Organization:
        """
        Get an organization by ID

        Parameters
        ----------
        organization_reference_id : str
            The reference ID of the organization.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Organization

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.organizations.get(
            organization_reference_id="organizationReferenceId",
        )
        """
        _response = self._raw_client.get(organization_reference_id, request_options=request_options)
        return _response.data

    def patch(
        self,
        organization_reference_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        default_language: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Organization:
        """
        Update mutable organization fields.
        All fields will overwrite the existing value on the organization only if provided.

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        organization_reference_id : str
            The reference ID of the organization.

        name : typing.Optional[str]
            The name of the organization.

        default_language : typing.Optional[str]
            The default language for the organization in ISO 639-1 code format.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Organization

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.organizations.patch(
            organization_reference_id="organizationReferenceId",
        )
        """
        _response = self._raw_client.patch(
            organization_reference_id, name=name, default_language=default_language, request_options=request_options
        )
        return _response.data

    def delete(
        self, organization_reference_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Delete an organization.

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        organization_reference_id : str
            The reference ID of the organization.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.organizations.delete(
            organization_reference_id="organizationReferenceId",
        )
        """
        _response = self._raw_client.delete(organization_reference_id, request_options=request_options)
        return _response.data

    def get_conversation_table(
        self,
        *,
        field_groupings: typing.Sequence[ConversationGroupBy],
        column_definitions: typing.Sequence[ConversationColumnDefinition],
        time_grouping: typing.Optional[TimeInterval] = OMIT,
        conversation_filter: typing.Optional[ConversationFilter] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationTableResponse:
        """
        Retrieves structured conversation data across all organizations, formatted as a table,
        allowing users to group, filter, and define specific metrics to display as columns.

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        field_groupings : typing.Sequence[ConversationGroupBy]
            Specifies the fields by which data should be grouped. Each unique combination forms a row.
            If multiple fields are provided, the result is grouped by their unique value combinations.
            If empty, all data is aggregated into a single row. |
            Note: The field `CreatedAt` should not be used here, all time-based grouping should be done using the `timeGrouping` field.

        column_definitions : typing.Sequence[ConversationColumnDefinition]
            Specifies the metrics to be displayed as columns. Column headers act as keys, with computed metric values as their mapped values. There needs to be at least one column definition in the table request.

        time_grouping : typing.Optional[TimeInterval]
            Defines the time interval for grouping data. If specified, data is grouped accordingly  based on the time they were created. Example: If set to "DAY," data will be aggregated by day.

        conversation_filter : typing.Optional[ConversationFilter]
            Optional filter applied to refine the conversation data before processing.

        timezone : typing.Optional[str]
            IANA timezone identifier (e.g., "America/Los_Angeles").
            When provided, time-based groupings (e.g., DAY) and date filters are evaluated in this timezone;
            otherwise UTC is used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationTableResponse

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.analytics import (
            ConversationColumnDefinition,
            ConversationGroupBy,
            ConversationMetric_Average,
            ConversationMetric_Count,
            ConversationMetric_Percentile,
        )
        from mavenagi.conversation import ConversationFilter

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.organizations.get_conversation_table(
            conversation_filter=ConversationFilter(
                languages=["en", "es"],
            ),
            time_grouping="DAY",
            field_groupings=[
                ConversationGroupBy(
                    field="Category",
                )
            ],
            column_definitions=[
                ConversationColumnDefinition(
                    header="count",
                    metric=ConversationMetric_Count(),
                ),
                ConversationColumnDefinition(
                    header="avg_first_response_time",
                    metric=ConversationMetric_Average(
                        target_field="FirstResponseTime",
                    ),
                ),
                ConversationColumnDefinition(
                    header="percentile_handle_time",
                    metric=ConversationMetric_Percentile(
                        target_field="HandleTime",
                        percentile=25.0,
                    ),
                ),
            ],
        )
        """
        _response = self._raw_client.get_conversation_table(
            field_groupings=field_groupings,
            column_definitions=column_definitions,
            time_grouping=time_grouping,
            conversation_filter=conversation_filter,
            timezone=timezone,
            request_options=request_options,
        )
        return _response.data

    def get_conversation_chart(
        self, *, request: ConversationChartRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> ChartResponse:
        """
        Fetches conversation data across all organizations, visualized in a chart format.
        Supported chart types include pie chart, date histogram, and stacked bar charts.

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        request : ConversationChartRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChartResponse

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.analytics import (
            ConversationChartRequest_PieChart,
            ConversationGroupBy,
            ConversationMetric_Count,
        )
        from mavenagi.conversation import ConversationFilter

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.organizations.get_conversation_chart(
            request=ConversationChartRequest_PieChart(
                conversation_filter=ConversationFilter(
                    languages=["en", "es"],
                ),
                group_by=ConversationGroupBy(
                    field="Category",
                ),
                metric=ConversationMetric_Count(),
            ),
        )
        """
        _response = self._raw_client.get_conversation_chart(request=request, request_options=request_options)
        return _response.data


class AsyncOrganizationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawOrganizationsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawOrganizationsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawOrganizationsClient
        """
        return self._raw_client

    async def create(
        self,
        organization_reference_id: str,
        *,
        name: str,
        default_language: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Organization:
        """
        Create a new organization.

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        organization_reference_id : str
            The reference ID of the organization.

        name : str
            The name of the organization.

        default_language : str
            The default language for the organization in ISO 639-1 code format.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Organization

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.organizations.create(
                organization_reference_id="organizationReferenceId",
                name="name",
                default_language="defaultLanguage",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            organization_reference_id, name=name, default_language=default_language, request_options=request_options
        )
        return _response.data

    async def get(
        self, organization_reference_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> Organization:
        """
        Get an organization by ID

        Parameters
        ----------
        organization_reference_id : str
            The reference ID of the organization.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Organization

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.organizations.get(
                organization_reference_id="organizationReferenceId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(organization_reference_id, request_options=request_options)
        return _response.data

    async def patch(
        self,
        organization_reference_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        default_language: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Organization:
        """
        Update mutable organization fields.
        All fields will overwrite the existing value on the organization only if provided.

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        organization_reference_id : str
            The reference ID of the organization.

        name : typing.Optional[str]
            The name of the organization.

        default_language : typing.Optional[str]
            The default language for the organization in ISO 639-1 code format.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Organization

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.organizations.patch(
                organization_reference_id="organizationReferenceId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.patch(
            organization_reference_id, name=name, default_language=default_language, request_options=request_options
        )
        return _response.data

    async def delete(
        self, organization_reference_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Delete an organization.

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        organization_reference_id : str
            The reference ID of the organization.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.organizations.delete(
                organization_reference_id="organizationReferenceId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(organization_reference_id, request_options=request_options)
        return _response.data

    async def get_conversation_table(
        self,
        *,
        field_groupings: typing.Sequence[ConversationGroupBy],
        column_definitions: typing.Sequence[ConversationColumnDefinition],
        time_grouping: typing.Optional[TimeInterval] = OMIT,
        conversation_filter: typing.Optional[ConversationFilter] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationTableResponse:
        """
        Retrieves structured conversation data across all organizations, formatted as a table,
        allowing users to group, filter, and define specific metrics to display as columns.

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        field_groupings : typing.Sequence[ConversationGroupBy]
            Specifies the fields by which data should be grouped. Each unique combination forms a row.
            If multiple fields are provided, the result is grouped by their unique value combinations.
            If empty, all data is aggregated into a single row. |
            Note: The field `CreatedAt` should not be used here, all time-based grouping should be done using the `timeGrouping` field.

        column_definitions : typing.Sequence[ConversationColumnDefinition]
            Specifies the metrics to be displayed as columns. Column headers act as keys, with computed metric values as their mapped values. There needs to be at least one column definition in the table request.

        time_grouping : typing.Optional[TimeInterval]
            Defines the time interval for grouping data. If specified, data is grouped accordingly  based on the time they were created. Example: If set to "DAY," data will be aggregated by day.

        conversation_filter : typing.Optional[ConversationFilter]
            Optional filter applied to refine the conversation data before processing.

        timezone : typing.Optional[str]
            IANA timezone identifier (e.g., "America/Los_Angeles").
            When provided, time-based groupings (e.g., DAY) and date filters are evaluated in this timezone;
            otherwise UTC is used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationTableResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.analytics import (
            ConversationColumnDefinition,
            ConversationGroupBy,
            ConversationMetric_Average,
            ConversationMetric_Count,
            ConversationMetric_Percentile,
        )
        from mavenagi.conversation import ConversationFilter

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.organizations.get_conversation_table(
                conversation_filter=ConversationFilter(
                    languages=["en", "es"],
                ),
                time_grouping="DAY",
                field_groupings=[
                    ConversationGroupBy(
                        field="Category",
                    )
                ],
                column_definitions=[
                    ConversationColumnDefinition(
                        header="count",
                        metric=ConversationMetric_Count(),
                    ),
                    ConversationColumnDefinition(
                        header="avg_first_response_time",
                        metric=ConversationMetric_Average(
                            target_field="FirstResponseTime",
                        ),
                    ),
                    ConversationColumnDefinition(
                        header="percentile_handle_time",
                        metric=ConversationMetric_Percentile(
                            target_field="HandleTime",
                            percentile=25.0,
                        ),
                    ),
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_conversation_table(
            field_groupings=field_groupings,
            column_definitions=column_definitions,
            time_grouping=time_grouping,
            conversation_filter=conversation_filter,
            timezone=timezone,
            request_options=request_options,
        )
        return _response.data

    async def get_conversation_chart(
        self, *, request: ConversationChartRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> ChartResponse:
        """
        Fetches conversation data across all organizations, visualized in a chart format.
        Supported chart types include pie chart, date histogram, and stacked bar charts.

        <Tip>
        This endpoint requires additional permissions. Contact support to request access.
        </Tip>

        Parameters
        ----------
        request : ConversationChartRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChartResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.analytics import (
            ConversationChartRequest_PieChart,
            ConversationGroupBy,
            ConversationMetric_Count,
        )
        from mavenagi.conversation import ConversationFilter

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.organizations.get_conversation_chart(
                request=ConversationChartRequest_PieChart(
                    conversation_filter=ConversationFilter(
                        languages=["en", "es"],
                    ),
                    group_by=ConversationGroupBy(
                        field="Category",
                    ),
                    metric=ConversationMetric_Count(),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_conversation_chart(request=request, request_options=request_options)
        return _response.data
